<?php
// $Id: entity_vocabulary.inc,v 1.4 2009/11/12 13:22:22 pounard Exp $

/**
 * @file
 * Vocabulary entity for Yamm
 */

/**
 * Simple user EntitySettings implementation
 */
class EntityVocabularySettings extends EntitySettings {

  const DUPLICATE_IGNORE = 1;
  const DUPLICATE_OVERRIDE = 2;

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/EntitySettings#settingsForm()
   */
  public function form() {
    $form = array();

    $options = array(
      EntityVocabularySettings::DUPLICATE_IGNORE => t('Create new vocabulary'),
      EntityVocabularySettings::DUPLICATE_OVERRIDE => t('Update existing'));

    $form['duplicate_behavior'] = array(
      '#type' => 'radios',
      '#title' => t('Duplicates handling'),
      '#options' => $options,
      '#description' => t('Define what behavior the the entity should adopt when vocabulary name conflicts with an existing one. Beware, if more than one duplicate is found, the algorithm won\'t merge but will create a new one instead.'),
      '#default_value' => $this->get('duplicate_behavior', EntityVocabularySettings::DUPLICATE_IGNORE));

    return $form;
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/EntitySettings#formValidate()
   */
  public function formValidate($values) {
    // Nothing to validate
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/EntitySettings#formSubmit($values)
   */
  public function formSubmit($values) {
    $this->set('duplicate_behavior', (int) $values['duplicate_behavior']);
  }
}

/**
 * Simple vocabulary Entity implementation
 */
class EntityVocabulary extends Entity {

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_objectLoad($identifier)
   */
  protected function _objectLoad($vid) {
    return taxonomy_vocabulary_load((int) $vid);
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_constructDependencies($object)
   */
  protected function _constructDependencies($vocabulary) {
    // No dependencies
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_save($object)
   */
  protected function _save($vocabulary) {
    $edit = (array) $vocabulary;
    unset($edit['vid']);
    if ($this->getSettings()->get('merge_duplicates', FALSE)) {
      $this->__mergeWithExisting($edit);
    }
    taxonomy_save_vocabulary($edit);
    return $edit['vid'];
  }

  private function __mergeWithExisting(&$edit) {
    if (1 != db_result(db_query("SELECT COUNT(*) FROM {vocabulary} WHERE name = '%s'", $edit['name']))) {
      return;
    }
    $edit['vid'] = db_result(db_query("SELECT vid FROM {vocabulary} WHERE name = '%s'", $edit['name']));
  }

  /**
   * (non-PHPdoc)
   * @see www/sites/all/modules/custom/yamm/yamm_api/Entity#_update($object, $identifier)
   */
  protected function _update($vocabulary, $vid) {
    $edit = (array) $vocabulary;
    $edit['vid'] = (int) $vid;
    taxonomy_save_vocabulary($edit);
  }
}
