<?php
// $Id: yamm_sync.job.inc,v 1.3 2009/11/05 20:19:36 pounard Exp $

/**
 * @file
 * Yamm client synchronization job implementation
 */

/**
 * Custom DataSync job implementation
 */
class yamm_sync extends datasync_job {

  public function phase_defs() {
    return array('syncAllContent');
  }

  public function config_defs() {
    $config = parent::config_defs();
    $config['phase_in_transaction'] = array('syncAllContent');
    return $config;
  }

  public function phase_proceed() {
    // Check we have a transaction in "running" state
    if ($this->_yammTid = yamm_client_transaction_get_running()) {

      // Also check server is configured
      if ($server_url = yamm_client_server_url()) {
        $this->_yammServerUrl = $server_url;
        return TRUE;
      }
      else {
        watchdog('yamm_client', 'You must configure server', array(), WATCHDOG_ERROR);
        $this->sendStatus(YAMM_TRANSACTION_STATUS_CANCELED);
        return FALSE;
      }
    }
    else {
      $this->sendStatus(YAMM_TRANSACTION_STATUS_CANCELED);
      return FALSE;
    }
  }

  public function postpone_phase() {
    $this->sendStatus(YAMM_TRANSACTION_STATUS_CANCELED);
  }

  public function fail_phase($method = NULL) {
    $this->sendStatus(YAMM_TRANSACTION_STATUS_CANCELED);

    return parent::fail_phase('fail_job');
  }

  public function sendStatus($status) {
    // Close transaction on client
    yamm_client_transaction_update_status($this->_yammTid, $status);

    // Notify server in case we have its url
    if ($this->_yammServerUrl) {
      $this->sendCall('yamm.client.status', $status);
    }
  }

  /**
   * Do an xmlrpc call to server
   *
   * @see yamm_api_xmlrpc_call()
   *
   * @param string $method
   *   Method to cal
   * @param ..
   *   Arbitrary parameters you want to send to
   * @return mixed
   *   Same result as yamm_api_xmlrpc_call()
   */
  public function sendCall($method) {
    if ($this->_yammServerUrl) {
      $args = func_get_args();
      $method = array_shift($args);
      array_unshift($args, $this->_yammTid);
      array_unshift($args, $method);
      array_unshift($args, $this->_yammServerUrl);
      return call_user_func_array('yamm_api_xmlrpc_call', $args);
    }
    return FALSE;
  }

  /**
   * Server URL
   * @var string
   */
  protected $_yammServerUrl = NULL;

  /**
   * Transaction identifier with server
   * @var string
   */
  protected $_yammTid = NULL;

  /**
   * One and only phase, content synchronisation
   */
  public function syncAllContent() {
    yamm_api_debug("Transaction " . $this->_yammTid . " running");
    yamm_api_bootstrap_entity();

    try {
      $parser = new EntityParser(
        new XmlrpcEntityFetcher(
          $this->_yammServerUrl, $this->_yammTid));

      $parser->parse();

      drupal_flush_all_caches();

      $this->sendStatus(YAMM_TRANSACTION_STATUS_FINISHED);
      return TRUE;
    }
    catch (EntityException $e) {
      // We have a problem here
      //   - first, DataSync does not store our exception backtrace, we have to
      //     do it here
      //   - second, DataSync will rollback our transaction, this means bye bye
      //     our backtrace!

      yamm_api_debug($e->getMessage());
      yamm_api_debug($e->getTraceAsString());

      // We dont have to change status here, fail_phase and others will do it
      // Pass the exception to DataSync, hoping it'll log it
      throw $e;
    }
  }
}